/*
 *
 *  kryptomedia- Another KDE cryto media application.
 *
 *  Copyright (C) 2006  Daniel Gollub <dgollub@suse.de>
 *
 *
 *  This file is part of kryptomedia.
 *
 *  kbluetooth is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  libkbluetooth is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with libkbluetooth; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "dialog.h"

Dialog *Dialog::_ctx = NULL;

Dialog::Dialog(QValueVector<KryptoMediaDevice*> *devices) :
	KDialogBase(NULL, "Dialog", true, "Decrypt Storage Device", (Cancel|User1), User1, false, KGuiItem(i18n("Decrypt"), "decrypted" ))
{
	_ctx = this;
	mDevices = devices;
	mDevice = devices->at(0)->DeviceNode;

	decryptDialog = new DecryptDialog(this);


	decryptDialog->errorBox->hide();
	decryptDialog->descLabel->setText(decryptDialog->descLabel->text().arg(devices->at(0)->Vendor)
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	  .arg(devices->at(0)->Product)
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	  .arg(devices->at(0)->DeviceNode));
	decryptDialog->descLabel->adjustSize();


	setDeviceIcon( devices->at(0)->DeviceType );

	enableButton( User1, false );
	
	if (devices->size() > 1) {
	 	 for (unsigned int i=0; i < devices->size(); i++) {
	 	 	 QListViewItem *tmp = new QListViewItem(decryptDialog->mediaListView, devices->at(i)->Vendor + devices->at(i)->Product,devices->at(i)->DeviceNode);
		 	 decryptDialog->mediaListView->insertItem(tmp);
	 	 }
	 	 decryptDialog->mediaListView->setSelected(decryptDialog->mediaListView->firstChild(), true);
	 	 decryptDialog->mediaListView->setCurrentItem(decryptDialog->mediaListView->firstChild());
	 	 setCurrentDevice(decryptDialog->mediaListView->firstChild());
	 	 connect(decryptDialog->mediaListView, SIGNAL (selectionChanged(QListViewItem*)), SLOT (setCurrentDevice(QListViewItem*)));
	} else 
	 	 decryptDialog->mediaBox->hide();

	connect(decryptDialog->passwordEdit, SIGNAL (textChanged(const QString &)), this, SLOT (slotPasswordChanged(const QString &)));

	setMainWidget(decryptDialog);
	decryptDialog->mediaListView->header()->hide();
	decryptDialog->adjustSize();
}

Dialog::~Dialog()
{
	delete decryptDialog;
}

void Dialog::setCurrentDevice(QListViewItem* cur)
{
	 QString devNode = cur->text(1);
	 for (unsigned int i=0; i <= mDevices->size(); i++) {
	 	 if (mDevices->at(i)->DeviceNode == devNode) {
	 	 	 decryptDialog->descLabel->setText(tr2i18n( "<p><b>%1 %2</b> (<b>%3</b>) is an encrypted storage device.</p>\n"
	 	 	 	 	 	 	 	 	 	 	 	 	 	 "<p>Please enter the password to decrypt the storage device.</p>" )
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	  .arg(mDevices->at(i)->Vendor)
                                                                      .arg(mDevices->at(i)->Product)
                                                                      .arg(mDevices->at(i)->DeviceNode));
	 	 	 setDeviceIcon( mDevices->at(i)->DeviceType );
		 	 mDevice = mDevices->at(i)->DeviceNode;
	 	 	 emit udiChanged(mDevices->at(i)->UDI);
	 	 	 break;
	 	 }
	 }

}


void Dialog::setDeviceIcon(QString deviceType)
{
	QString deviceIcon;

	if ( deviceType == "memory_stick"
		|| deviceType == "cdrom"
		|| deviceType == "sd_mmc"
		|| deviceType == "compact_flash"
		|| deviceType == "smart_media"
		|| deviceType == "zip" ) {

		deviceIcon = QString("%1_unmount").arg(deviceType);
	} else {
		deviceIcon = QString("hdd_unmount");
	}

	QPixmap pixmap = KGlobal::iconLoader()->loadIcon(deviceIcon, KIcon::NoGroup, KIcon::SizeLarge);
	decryptDialog->encryptedIcon->setPixmap( pixmap );
}

QString Dialog::getPassword()
{
	return decryptDialog->passwordEdit->text();
}

void Dialog::slotDialogError(QString errorName, QString errorMsg)
{
	kdDebug() << __func__ << "(" << errorName << ", " << errorMsg << " )" << endl;

	QString error = QString::null;

	if ( errorName == "org.freedesktop.Hal.Device.Volume.Crypto.SetupPasswordError" ) {
		error = QString( i18n("Wrong password!") );
	} else if ( errorName == "org.freedesktop.Hal.Device.Volume.Crypto.CryptSetupMissing" ) {
		error = QString( i18n("Decryption failed! Application \"cryptsetup\" not found. Is package \"util-linux-crypto\" installed?") );
	} else if ( errorName == "org.freedesktop.Hal.Device.Volume.Crypto.SetupError" ) {
		error = QString( i18n("%1 is already decrypted!").arg(mDevice) );
	} else {
		error = errorMsg;
	}

	decryptDialog->errorLabel->setText(QString("<b>%1</b>").arg(error));
	decryptDialog->errorBox->show();
	adjustSize();
	decryptDialog->errorBox->adjustSize();
	resize(width(),height() + 2);
	
}

void Dialog::slotPasswordChanged(const QString &text)
{
	enableButton( User1, !text.isEmpty() );
}

#include "dialog.moc"
